	page	240, 132
;OCTDEMO.ASM	10-SEP-2001
;Tumbling Octahedron Demo in 1K
; by Lenny Boreal
;
;Dedicated to The 6502 Group.
;
;Inspired by Richard Willmore and some really amazing demos available at:
; http://www.hornet.org
;
;Images are built in numbuf and scnbuf then blitted to video RAM. This not
; only solves the flicker problem, but is actually faster than plotting a
; pixel at a time directly to video RAM (because it writes two pixels at 
; once and it avoids clearing video RAM).
;
;A significant effort was made to squeeze this code into 1K. Code squeezing is
; a symptom of "excessive cleverness" that unfortunately results in programs
; that are difficult to understand and modify. Sorry.
;
;Assemble with:
; tasm /m octdemo
; tlink /t octdemo
;
;TASM 3.1 is required (rather than MASM 5.10) to preserve the 1024-byte size.
; (TASM doesn't increase .COM file size for "dw ?" at end, whereas MASM does.)
;
;If you have any feedback, I'd really like to hear it. You can reach me c/o
; Loren Blaney at: loren_blaney@idcomm.com
; Thanks!
;
	.386
cseg	segment dword public use16 'code'
	assume	cs:cseg, ds:cseg, es:cseg, ss:cseg

;Other registers are assumed to be initialized by the loader. This program
; will not run under a debugger such as Turbo Debugger 3.1, which zeros these
; registers. It also will not run under FreeDOS 0.72, which also zeros these
; registers. Standard DOS loaders initialize these registers as follows:
;  ax = 0000
;  bx = 0000
;  cx = 00FF
;  dx = cseg
;  si = 0100
;  di = FFFE
;  bp = 09??
;  sp = FFFE
;  fs = ????
;  gs = ????
;
;The direction flag (d) is assumed to be clear (incrementing).
;
;There is assumed to be RAM available at 70000h thru 8FFFFh.
;===============================================================================

	org	100h			;this is a COM file so skip the PSP
start:	mov	al, 13h			;set mode 13h graphics (320x200x256)
	int	10h			; & clear screen (some BIOSes clear al)

	mov	bp, offset base		;set base address for accessing data

;Change color registers 1eh-22h from bright white to gray to reduce glitches
	mov	dx, 3C8h		;address to select color register
	mov	al, 1Eh			;color register number
;	cli				;beware of Windows simultaneously
	out	dx, al			; accessing color registers
	inc	dx			;= 3c9h = address to write to color regs
	mov	al, 40			;a nice shade of gray
	mov	cl, 5*3			;change 5 sets of color registers: R,G,B
strt00:	out	dx, al			; (ch=0)
	loop	strt00			; cx--
;	sti				;reenable interrupts

;Save "colregs" number of color registers into paltbl (used for fade-out at end)
	mov	ax, 1017h		;BIOS function: get block of color regs
;	xor	bx, bx			;first color register (done by loader)
	mov	cl, colregs		;number of color registers (ch=0)
	mov	dx, offset paltbl	;address of table to receive data
	int	10h			;(es=ds)

	push	scnbuf			;clear scnbuf (do it now because of
	pop	es			; blit in pi display)
	call	clrscn

	push	numbuf			;clear the number buffer
	pop	es
	xor	di, di			;point to first word to clear
	xor	ax, ax
	mov	cx, 32768		;number of bytes to clear
	rep stosw			;es:[di++] <- ax  cx--
					;(ax=bx=cx=di=0, es=numbuf)
;Set up coroutine stack to look like a suspend occurred
	push	offset pi00		;push entry loc for alternate coroutine
	pusha				;do suspnd's register save
	push	es
	mov	[bp+spsave-base], sp	;save alternate coroutine's stack pntr
	sub	sp, 126			;fall into first coroutine...

;===============================================================================
;Task to draw a circle, inscribe lines, then draw a tumbling octahedron

	mov	ax, 17*60		;wait awhile before drawing the circle
	call	delay			;(let the pi coroutine run awhile)
					;(ax=0)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Draw a circle

c0x	equ	160			;coordinates of center of circle
c0y	equ	99
r0	equ	50			;radius (in pixels)
lcolor	equ	1Eh			;line color: gray, one-pixel wide

;Register usage:
; ax = scratch
; bx = X coordinate
; cx = loop counter
; dx = Y coordinate
; di = pointer to memory
; es = scnbuf

	push	scnbuf			;draw on scnbuf
	pop	es

	mov	bx, r0*256+63		;X:= R0*256+63 (63 makes circle rounder)
	cwd				;Y:= 0*256 (ax=0 so dx:= 0)
	mov	cx, 402			;for T:= 1, 402 do...
cir10:	mov	ax, dx			;X:= X + Y/64
	sar	ax, 6			;sar is not the same as a divide and it
	add	bx, ax			; causes some slight distortion
	mov	ax, bx			;Y:= Y - X/64
	sar	ax, 6
	sub	dx, ax

;Point(X/256+C0X, Y/256+C0Y, $F)
	mov	ax, c0y			;di:= X/256+C0X + (Y/256+C0Y)*320
	add	al, dh
	imul	di, ax, 320
	mov	al, bh			;stretch the X axis to compensate for
	cbw				; distorted aspect ratio
	imul	ax, 19
	sar	ax, 4			;/16
	add	ax, c0x			;+center
	add	di, ax
	mov	al, lcolor		;es:[di++]:= lcolor
	stosb				;plot point

	call	suspnd			;run the coroutine awhile
	loop	cir10			;loop for all the pixels in the circle
	call	delay2s			;pause to admire result

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Inscribe lines in the circle. The *19/16 insanity compensates for aspect ratio.

	mov	al, lcolor		;slowly draw horizontal diameter
	mov	di, c0y*320+c0x-(r0*19/16) ;point to left end of line on scnbuf
	mov	cl, (r0+r0+1)*19/16	;(ch = 0 from loop instruction above)
seg00:	stosb				;es:[di++]:= lcolor
	call	suspnd
	loop	seg00			;cx--
	call	delay2s			;ponder significance (hint: pi = C/D)

;Move(C0X, C0Y+R0): lineX,lineY have been preset by loader
;The following assumes that the high bytes in cx and dx are clear
	mov	si, offset segtbl	;point to start of table
seg10:	lodsw				;ax:= ds:[si++]
	mov	cl, al
	mov	dl, ah
	mov	ax, lcolor
	call	line
	call	delay30
	cmp	si, offset segend	;loop for all segments in table
	jb	seg10
	call	delay2s			;pause before erasing circle & tumbling
					;(ax=0)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;Draw a tumbling octahedron

edges	equ	13			;number of edges +1 (for starting point)
alt	equ	4242			;altitude (scaled up to reduce errors)
spinZ	equ	530			;initial rotation speed about Z & X axes
spinX	equ	300			; larger values rotate slower

;Initialize octahedron arrays (ax=0)
	mov	bx, bp			;point to arrayX
	mov	cl, edges*3		;loop for all 3 arrays (ch=0)
init10:	mov	[bx], ax		;clear array location
	bt	[bp+tbl1-base], cx	;is there an alt here?
	jnc	init20			;jump if not
	mov	word ptr [bx], alt	;store alt into array
	bt	[bp+tbl2-base], cx	;is alt negative?
	jnc	init20			;jump if not
	neg	word ptr [bx]		;negate alt
init20:	inc	bx			;next array location
	inc	bx
	loop	init10			;next bit in tbls

;Draw the octahedron
; Register usage:
;  ax = scratch
;  bx = array index
;  cx = X line end point
;  dx = Y line end point
;  si = magnification factor
;  di = loop counter (edges)

	mov	si, alt/r0*8		;initialize magnification factor
draw00:	call	clrscn			;clear scnbuf

;MOVE(X(0)/SIZ+160, 99-Y(0)/SIZ)
	mov	bx, bp			;initialize array index
	push	si			;save large magnification factor
	shr	si, 3			;make small magnification factor (/8)

	call	draw50			;set up cx & dx for end point of line
	mov	[bp+lineX-base], cx
	mov	[bp+lineY-base], dx

;for I:= 1, EDGES-1 do LINE(X(I)/SIZ+160, 99-Y(I)/SIZ, COLOR)
	mov	di, edges-1		;for all the edges...
draw10:	inc	bx			;next index (skip starting point)
	inc	bx

	call	draw50			;set up cx & dx for end point of line

;Set line thickness as a function of (small) magnification factor
;alt/50=84, alt/90=47, 84-47=37, 37/7=5, 84-3=81
	push	dx			;save dx
	mov	ax, 81			;thickness:= (81-magnification)/7
	sub	ax, si			; (gives 0..4)
	mov	dl, 7
	cbw				;ax:= extend(al)
	idiv	dl			;al:= ax/dl
	xchg	dx, ax			;save thickness code in dl

;Set color as a function of the Z distance of the mid point of the line
	mov	ax, [bx+arrayZ-base-2]	;previous point's Z distance
	add	ax, [bx+arrayZ-base]	;current point's Z distance
	sar	ax, 11			;div by 2048
	neg	ax			;make oct appear to tumble opposite dir
	add	al, 19h			; so it rolls with the numbers

	mov	ah, dl			;get thickness
	pop	dx			;restore dx
	call	line			;draw line with color and thickness

	dec	di			;next line end point
	jne	draw10			;loop

;Rotate the octahedron
	mov	cx, edges		;for I:= 0, EDGES-1 do...
	mov	bx, bp			;initialize array index
	mov	si, spinZ		;set up rotation-speed divisor
spinZ1:					;self-modifying code
	mov	di, spinX		;set up rotation-speed divisor
spinX1:					;self-modifying code
rot10:	mov	ax, [bx+arrayY-base]	;rotate about Z axis
	cwd				;X(I):= X(I) + Y(I) /C1
	idiv	si			;ax:= ax:dx/si
	add	[bx+arrayX-base], ax

	mov	ax, [bx+arrayX-base]	;Y(I):= Y(I) - X(I) /C1
	cwd
	idiv	si			;ax:= ax:dx/si
	sub	[bx+arrayY-base], ax

	mov	ax, [bx+arrayZ-base]	;rotate about X axis
	cwd				;Y(I):= Y(I) + Z(I) /C2
	idiv	di			;ax:= ax:dx/di
	add	[bx+arrayY-base], ax

	mov	ax, [bx+arrayY-base]	;Z(I):= Z(I) - Y(I) /C2
	cwd
	idiv	di			;ax:= ax:dx/di
	sub	[bx+arrayZ-base], ax

	inc	bx			;next index
	inc	bx
	loop	rot10			;loop for all end points (vertices)

	cmp	si, 5			;increase rotation speed slightly
	je	rot20
	dec	word ptr [spinZ1-2]
rot20:	cmp	di, 12
	je	rot25
	dec	word ptr [spinX1-2]
rot25:

;Show the result
; ("delay" calls "suspnd", which executes pi coroutine, which executes "blit")
	mov	ax, 3			;wait 3 vertical blanks (1/20 sec)
	call	delay

	pop	si			;restore large magnification factor
	dec	si			;gradually increase size of octahedron
	cmp	si, alt/90*8		; (by decrementing the divisor/8)
	jae	rot30			;limit maximum size
	inc	si			;undo dec
rot30:
	jmp	draw00			;loop

;-------------------------------------------------------------------------------
;Common code for drawing octahedron.
; Inputs: bx
; Outputs: cx, dx, the end points of the line to draw
; LINE(X(I)/SIZ+160, 99-Y(I)/SIZ, COLOR)

draw50:	mov	ax, [bx+arrayX-base]	;get X coordinate
	cwd				;ax:= ax:dx/si
	idiv	si			;divide by (small) magnification factor
	imul	ax, 19			;compensate aspect ratio
	sar	ax, 4			; (/16)
	add	ax, 160			;add offset to center of screen (=320/2)
	xchg	cx, ax			;cx:= ax

	mov	ax, [bx+arrayY-base]	;do same with Y coordinate
	cwd
	idiv	si			;ax:= ax:dx/si
	neg	ax
	add	ax, 99
	xchg	dx, ax			;dx:= ax
	ret

;-------------------------------------------------------------------------------
;Delay ax vertical blanks (= ax/60 seconds). Range of ax: 0..32767 (9.1 minutes)
; Note that there are multiple entry points to this routine.
; Register ax = 0 upon return.

delay2s:call	delay1s			;delay 2.0 second
delay1s:call	delay30			;delay 1.0 second
delay30:mov	ax, 30-1		;delay 0.5 second
delay:	call	suspnd			;(pi coroutine has waitVB in its loop)
	dec	ax
	jne	delay
	ret

;-------------------------------------------------------------------------------
;Draw a line from lineX, lineY to cx, dx using color in al and thickness in ah.
; Also inputs es = scnbuf.
; LineX and lineY are updated to the new end point coordinate in cx, dx.
; (There is no clipping at the edge of the screen; it wraps instead.)
; Register usage:
;  cx = DeltaX		abs(cx-lineX)
;  dx = DeltaY		abs(dx-lineY)
;  si = IncX, NE	plotting increment, NorthEast (diagonal move)
;  di = IncY, SD	plotting increment, Step Ptr Diagonally
;  ax = Dir		direction to move: adjacent or diagonal
;  bx = Ptr		pointer to display RAM
;  dx = E		adjacent move (East)
;  bp = S		Step Ptr adjacently (east)
;  es = scnbuf

line:	pusha				;save registers

	mov	byte ptr [color1-1], al	;set up color using self-modifying code

	mov	al, 4			;set up line thickness
	sub	al, ah			;[line71-1]:= (4-ah) * 2 * 5 bytes/instr
	cbw
	imul	ax, 10
	mov	byte ptr [line71-1], al	;self-modifying code

	imul	bx, [bp+lineY-base], 320 ;Ptr:= ScreenBuf + LineY*Width + LineX
	add	bx, [bp+lineX-base]	;bx points to first pixel on line     bx
	mov	ax, cx			;DeltaX:= cx - LineX
	sub	ax, [bp+lineX-base]
	mov	[bp+lineX-base], cx	;end of line is beginning of next line
	xchg	cx, ax			;cx = DeltaX			      cx

	mov	si, 1			;if DeltaX >= 0 then IncX:= 1         si
	jns	line10
	neg	cx			;else [DeltaX:= -DeltaX;   IncX:= -1]
	neg	si
line10:
	mov	ax, dx			;DeltaY:= dx - LineY
	sub	ax, [bp+lineY-base]
	mov	[bp+lineY-base], dx	;end of line is beginning of next line
	xchg	dx, ax			;dx = DeltaY			      dx

	mov	di, 320			;if DeltaY >= 0 then IncY:= Width     di
	jns	line20
	neg	dx			;else [DeltaY:= -DeltaY; IncY:= -Width]
	neg	di
line20:
	mov	bp, si			;S:= IncX			      bp
	mov	ax, di			;line thickness increment (+/-320)
	cmp	cx, dx			;if DeltaX < DeltaY then
	jns	line40			;
	mov	bp, di			;	S:= IncY
	xchg	cx, dx			;	swap DeltaX, DeltaY
	mov	ax, si			;line thickness increment (+/-1)
line40:
;Set up code to draw thick lines with color gradients
; Register usage:
;  ax = step (step:= step + inc)
;  bx = color
;  cx = loop counters
;  dx = inc (+/-1 or +/-320)
;  di = pointer to modified code
	pusha				;save registers
	mov	dx, ax			;set up inc (= +/-1 or +/-320)
	mov	bl, byte ptr [color1-1]	;get initial color (for center of line)
	mov	di, offset color2-1	;point to first location to modify
	mov	cl, 4			;loop for thickness/2
line43:	mov	ch, 2			;loop
	dec	bx			;next color
line45:	mov	[di], bl		;insert color into code
	mov	[di-2], ax		;insert step size for thick lines
	neg	ax			;do other side of thick line
	sub	di, 5			;next code location
	dec	ch			;loop
	jne	line45
	add	ax, dx			;increase step size by inc
	loop	line43			;loop for thickness (ch = 0)
	popa				;restore registers

;Set up decision variables
	add	di, si			;SD:= IncX + IncY
	mov	si, dx			;NE:= 2*(DeltaY-DeltaX)
	sub	si, cx
	add	si, si
	add	dx, dx			;E:= 2*DeltaY
	mov	ax, dx			;Dir:= 2*DeltaY - DeltaX
	sub	ax, cx
	inc	cx			;add a pixel for the first point

	push	ds
	push	es			;ds:= scnbuf
	pop	ds
	jmp	line70			;go plot line, also flush prefetch queue
					; for self-modifying code
line50:	test	ax, ax			;if Dir < 0 then move adjacently
	jns	line60
	add	bx, bp			;	Ptr:= Ptr + S
	add	ax, dx			;	Dir:= Dir + E
	jmp	line70			;else move diagonally
line60:	add	bx, di			;	Ptr:= Ptr + SD
	add	ax, si			;	Dir:= Dir + NE
line70:	jmp	line71			;jump according to specified thickness
line71:					;jmp location is modified by code
	or	byte ptr [bx+1280], 0	;ORing makes light lines appear in
	or	byte ptr [bx-1280], 0	; front of dark lines
	or	byte ptr [bx+960], 0	;0's are modified by code
	or	byte ptr [bx-960], 0
	or	byte ptr [bx+640], 0
	or	byte ptr [bx-640], 0
	or	byte ptr [bx+320], 0
	or	byte ptr [bx-320], 0
color2:					;self-modifying code, 256 colors
	or	byte ptr [bx], 0	;Ptr(0):= Color; Plot point
color1:					;self-modifying code, 256 colors
	loop	line50			;loop for all pixels along long axis

	pop	ds			;restore registers
	popa
	ret

;===============================================================================
;Task to slowly display pi = 3.14159....
; Register usage:
;  ax = scratch
;  bl = high byte of Velocity
;  bh = 0
;  cx = distance (X), actually time duration between displaying digits
;  dx = velocity (V), actually frequency that digits are displayed
;  si = digit counter = index into pistr
;  di = acceleration (A)
;  es = numbuf
;
pi00:					;(ax=bx=cx=di=0, es=numbuf)
;	xor	di, di			;A:= 0;		(already done)
;	xor	bx, bx			;V:= 4096;	(done by loader)
	mov	dx, 4096
;	xor	cx, cx			;X:= 0;		(already done)
	xchg	si, ax			;index:= 0; si:= 0

pi10:	inc	di			;A:= A + 1, increase acceleration
	mov	ax, di			;V:= V + A/4
	shr	ax, 2
	add	dx, ax
	adc	bl, bh			;+0
	add	cl, dh			;X:= X + V/256;
	adc	ch, bl
	je	pi50			;jump if high byte of X = 0

;Return a random number between 0 and 9
;Chout(0, Ran(10)+^0);
pi30:	imul	ax, [bp+seed-base], 9421 ;seed:= (seed*9421 + 1) modulo 65536
	inc	ax			 ;"Master Class Assembly Language" p 838
	mov	[bp+seed-base], ax

	push	bx			;modulo 10 (remainder of seed/10)
	push	dx
	xor	dx, dx
	mov	bl, 10			;(bh=0)
	idiv	bx			;ax:= dx:ax/bx  dx:= remainder
	xchg	ax, dx			; (also clears ah)
	pop	dx
	pop	bx

	cmp	si, pilen		;there are only pilen items in pistr
	jae	pi40			;jump if we're out of correct digits
	mov	al, [bp+si+pistr-base]	;get correct digit from string
pi40:

;Display the character (0..13) in al at the current cursor position.
; Register usage:
;  al = Ch
;  ah = 0
;  bx = font index
;  cl = I
;  ch = J
;  dl = Bit
;  dh = Col (column of pixels)
;  di = plot address
;  es = numbuf

	pusha				;save registers

	imul	bx, ax, 5		;font index, bx:= Ch*5 (ah=0)

;if cursor >= 25*45 then [Scroll;  cursor:= 24*45]
	mov	ax, [bp+cursor-base]
	cmp	ax, 25*45
	jl	plch05

;Scroll up one line
	push	ds
	push	es			;ds:= numbuf
	pop	ds
	mov	si, 8*320		;one text line down
	xor	di, di
	mov	cx, 320*8*24/2		;number of words to move
	rep movsw			;es:[di++] <- ds:[si++]  cx--
	pop	ds

;Clear bottom line (di=bottom line)
	xchg	ax, cx			;ax:= 0
	mov	cx, 320*8/2		;number of words to clear
	rep stosw			;es:[di++] <- ax  cx--

	mov	ax, 24*45
plch05:	inc	ax			;advance cursor
	mov	[bp+cursor-base], ax	;save cursor position
	dec	ax			;restore current position

	mov	cx, 45			;number of character cells per line
	cwd				;dx:= 0
	idiv	cx			;ax:= dx:ax/cx
	imul	ax, 320*8		;quotient * num bytes / char cell row
	imul	di, dx, 7		;remainder * num bytes / char cell
	add	di, ax			;di points to upper left pixel of char

	mov	cl, 5			;for I:= 0, 4 do
plch10:	push	di			;  save top of pixel column (on screen)
	mov	dh, [bx+font]		;  Col:= Font(bx)  get column of pixels
	mov	dl, 01h			;  Bit:= $01
	mov	ch, 7			;  for J:= 0, 6 do
plch20:	test	dh, dl			;    if Col & Bit then
	je	plch30			;    Point(CursorX+I, CursorY+J, $A)
	mov	byte ptr es:[di], 02h	;    Point(Cursor+I+320*J):= 2 (green)
plch30:	add	dl, dl			;    Bit:= Bit + Bit
	add	di, 320			;  next pixel row down
	dec	ch			;  next J
	jne	plch20

	pop	di			;move to top of next pixel column
	inc	di
	inc	bx			;next font column
	loop	plch10			;next I (ch=0)

	popa

	inc	si			;bump index

	dec	ch			;X:= X - 256 account for displayed digit
	jne	pi30			;loop
pi50:					;(ch=0)
	call	suspnd			;execute other coroutine

;Combine numbuf and scnbuf and copy to video display memory (at 0A000h)
;numbuf + scnbuf -> video ram
	pusha				;save registers
	push	ds

	push	0A000h			;set up segment registers to point to
	pop	fs			; respective buffers
	push	scnbuf
	pop	ds

	mov	di, 64000-2		;point to last words in buffers

blit10:	mov	ax, [di]		;fetch two bytes from scnbuf
	add	ax, es:[di]		;combine with two bytes from numbuf
	mov	fs:[di], ax		;store into video ram
	dec	di			;loop for entire screen
	dec	di
	jne	blit10

;Wait for beginning of next vertical blank (stolen from Picard / Hydrogen)
	mov	dx, 3DAh
	db	0B4h			;mov ah,
wait10:	lahf				;ah bit 2 = parity flag = ~last retrace
	in	al, dx
	and	ax, 0408h
	jne	wait10			;until ~current retrace & last retrace

	pop	ds			;restore registers
	popa

;Fade out and exit when demo is over.
;Step intensity to white (63) then step it back to black (0).
	test	si, si			;have more than 32767 digits been done?
	jns	fade30			;jump if not--skip fade

	pusha				;save registers
	mov	si, offset paltbl	;point to table of original colors
	push	si			;save for later
	mov	cl, colregs*3		;number of bytes in table (ch=0)
fade20:	mov	al, [si]		;fetch color
	add	al, 4			;adjust intensity (modified by code)
fade21:	jns	fade22			;clip intensity to 0..63 range
	mov	al, 0
fade22:	cmp	al, 63
	jle	fade23
	mov	al, 63
fade23:	mov	[si], al		;store adjusted intensity back in table
	inc	si			;next color
	loop	fade20

	pop	si			;si = paltbl
	cmp	byte ptr [si], 63	;is 1st color (black) at max intensity?
	jl	fade24			;skip if not
	neg	byte ptr [fade21-1]	;else reverse direction (white to black)
fade24:

;Set color registers
	mov	cl, colregs*3		;number of registers to set (ch=0)
	xor	ax, ax			;start with register 0
	mov	dx, 3C8h		;select color register to write
	out	dx, al
	inc	dx			;point to color reg data
	rep outsb			;[dx] <- ds:[si++], cx--

	popa				;restore CPU registers

	cmp	[bp+paltbl-base], bh	;when black is back to 0 then exit
	je	pi90
fade30:
	mov	ah, 01h			;loop until keystroke (or fade-out)
	int	16h
	je	pi10

	xchg	ax, bx			;eat keystroke; ah:= 0
	int	16h

pi90:	mov	ax, 0003h		;return to text mode (assume mode 03h)
	int	10h

	mov	cl, 8
	mov	al, 0Ah			;move cursor to start of 9th line
pi93:	int	29h
	loop	pi93

	push	0B800h			;display green '6502...' in upper left
	pop	es			; corner
	mov	cl, 10			;number of characters (ch=0)
	mov	si, offset endstr
	xor	di, di
	mov	ah, 2			;green
pi95:	lodsb				;al:= ds:[si++]
	stosw				;es:[di++]:= ax
	loop	pi95			;cx--
	ret				;return to DOS via int 20h in PSP

;===============================================================================
;Clear scnbuf, the screen buffer
; Inputs: es = scnbuf

clrscn:	pusha				;save registers

	xor	di, di			;point to first word to clear
	xor	ax, ax
	mov	cx, 32000		;number of words to clear
	rep stosw			;es:[di++] <- ax  cx--

	popa
	ret

;-------------------------------------------------------------------------------
;Suspend the current coroutine and resume the alternate coroutine
;
suspnd:	pusha				;save current coroutine's registers
	push	es
	xchg	[bp+spsave-base], sp	;save current sp and get alternate sp
	pop	es			;restore alternate coroutine's registers
	popa
	ret

;===============================================================================
;Preset variables and tables go here, after the code. The bp register, which is
; set to "base", is used to access these variables with a 1-byte offset instead
; of using the normal 2-byte address. Some variables are loaded with values to
; avoid initializing them in the code.
;
font	db	3Eh, 51h, 49h, 45h, 3Eh	;0
	db	00h, 42h, 7Fh, 40h, 00h	;1  MSB           LSB
	db	62h, 51h, 49h, 49h, 46h	;2   . * * . . . * .
	db	21h, 41h, 49h, 4Dh, 33h	;3   . * . * . . . *
	db	18h, 14h, 12h, 7Fh, 10h	;4   . * . . * . . *
	db	27h, 45h, 45h, 45h, 39h	;5   . * . . * . . *
	db	3Ch, 4Ah, 49h, 49h, 31h	;6   . * . . . * * .
	db	01h, 71h, 09h, 05h, 03h	;7
	db	36h, 49h, 49h, 49h, 36h	;8
	db	46h, 49h, 49h, 29h, 1Eh	;9
	db	00h, 00h, 40h, 00h, 00h	;.	10
	db	14h, 14h, 14h, 14h, 14h	;=	11
	db	04h, 7Ch, 04h, 7Ch, 04h	;pi	12
	db	00h, 00h, 00h, 00h, 00h	;space	13

cursor	dw	12*45+2			;start on middle line indented a little

;String of the initial pi digits. The leading space character provides time for
; the monitor to settle after changing video modes. Adjust this string to make
; the program exactly 1024 bytes (1K) long.
; (pi = 3.14159 26535 89793 23846 26433 83279...)
pistr	db	13, 12, 13, 11, 13, 3, 10		;"pi = 3."
	db	1, 4, 1, 5, 9, 2, 6, 5, 3, 5, 8, 9, 7, 9, 3
	db	2, 3, 8, 4, 6, 2, 6, 4, 3, 3

pilen	equ	32			;number of characters in pistr

endstr	db	'...6502...'

lineX	dw	c0x			;coordinate of end of last line drawn
lineY	dw	c0y+r0			; = beginning of next line

;Table of X,Y coordinates used to draw the line segments inscribed in the circle
; (The *19/16 insanity compensates for aspect ratio.)
segtbl	db	c0x, c0y-r0
	db	c0x-(r0*19/16), c0y
	db	c0x, c0y+r0
	db	c0x+(r0*19/16), c0y
	db	c0x, c0y-r0
segend	equ	$

;Bits indicate where "alt" is stored into arrays X, Y, Z (see arrayX below)
tbl1	dw	0100001001010100b, 1000010100100001b
	db	01010010b
tbl2	dw	0100000001000100b, 0000010000100000b	;-alt
	db	00010010b

seed	dw	12343			;initial random number seed

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

base	equ	arrayX			;= bp register; used to reduce code size

;The octahedron is drawn as a continuous sequence of line segments starting at A
;This is how the initialization code sets up the arrays:
;		 A    B    C    D    E    A    D    F    B    E    C    F    A
;arrayX	dw	 0,   alt, 0,  -alt, 0,   0,  -alt, 0,   alt, 0,   0,   0,   0
;arrayY	dw	-alt, 0,   alt, 0,   0,  -alt, 0,   0,   0,   0,   alt, 0,  -alt
;arrayZ	dw	 0,   0,   0,   0,   alt, 0,   0,  -alt, 0,   alt, 0,  -alt, 0

arrayX	dw	 ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?
arrayY	dw	 ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?
arrayZ	dw	 ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?

spsave	dw	?			;save stack pointer for coroutines

;This table is used for the fade-out at the end
colregs	equ	23h			;number of color registers to fade
paltbl	db	colregs*3 dup (?)	;initial color register values

numbuf	equ	7000h			;segment address for graphic numbers
scnbuf	equ	8000h			;segment address for screen buffer

cseg	ends
	end	start
